//------------------------------------------------------------------------------------------------
//
//   Project Blue - Pneumatic Extractor part
//
//------------------------------------------------------------------------------------------------

package gcewing.projectblue;

import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.entity.player.*;
import net.minecraft.inventory.*;
import net.minecraft.item.*;
import net.minecraft.nbt.*;
import net.minecraft.tileentity.*;
import net.minecraft.util.*;

import codechicken.lib.data.*;
import codechicken.lib.vec.Cuboid6;
import codechicken.multipart.*;

import mrtjp.projectred.api.*;
import mrtjp.projectred.transmission.*;

import static gcewing.projectblue.BaseUtils.*;

//public class PneumaticExtractorPart extends PBFacePart implements IFaceRedstonePart {
public class PneumaticExtractorPart extends PBWiredFacePart implements IRedwireEmitter {

	static Cuboid6[] bounds = newBounds(3/16d, 0, 3/16d, 13/16d, 5/16d, 13/16d);
	static PBTexture[] textures;
	static PBModel model;
	
	public int interval = 20; // Ticks between extractions
	public int minStack = 1;
	public int maxStack = 64;
	public int tag = -1; // Tag colour, -1 for no tag
	public boolean redstoneInput;

	@Override
	public Cuboid6[] getBoundsArray() {
		return bounds;
	}
	
	@Override
	public void writeDesc(MCDataOutput data) {
		super.writeDesc(data);
		data.writeInt(minStack);
		data.writeInt(maxStack);
		data.writeInt(interval);
		data.writeInt(tag);
	}

	@Override
	public void readDesc(MCDataInput data) {
		super.readDesc(data);
		minStack = data.readInt();
		maxStack = data.readInt();
		interval = data.readInt();
		tag = data.readInt();
	}

	@Override
	public void save(NBTTagCompound nbt) {
		super.save(nbt);
		nbt.setInteger("minStack", minStack);
		nbt.setInteger("maxStack", maxStack);
		nbt.setInteger("interval", interval);
		nbt.setInteger("tag", tag + 1);
	}

	@Override
	public void load(NBTTagCompound nbt) {
		super.load(nbt);
		minStack = nbt.getInteger("minStack");
		maxStack = nbt.getInteger("maxStack");
		interval = nbt.getInteger("interval");
		tag = nbt.getInteger("tag") - 1;
	}

	@Override
	public void onAdded() {
		if (!world().isRemote) {
			scheduleExtraction();
		}
	}
	
	void scheduleExtraction() {
		scheduleTick(interval);
	}
	
	@Override
	public void scheduledTick() {
		performExtraction();
		scheduleExtraction();
	}
	
	void performExtraction() {
		PneumaticTubePart tube = getTube();
		if (tube != null && tube.willAcceptFromSide(side)) {
			TileEntity te = getAdjacentTileEntity();
			if (te instanceof IInventory) {
				ItemStack stack = extractFromInventory((IInventory)te, side ^ 1);
				if (stack != null && stack.stackSize > 0) {
					System.out.printf("PneumaticExtractorPart.scheduledTick: %s: inserting stack with tag %s\n",
						this, tag);
					tube.insertFromSide(side, stack, tag);
				}
			}
		}
	}
	
	PneumaticTubePart getTube() {
		TMultiPart part = tile().partMap(6);
		if (part instanceof PneumaticTubePart)
			return (PneumaticTubePart)part;
		else
			return null;
	}
	
	ItemStack extractFromInventory(IInventory te, int side) {
		//System.out.printf("PneumaticExtractorPart.extractFromInventory: side %s of %s\n", side, te);
		InventorySide inv = inventorySide(te, side);
		for (int i = 0; i < inv.size; i++) {
			ItemStack stack = inv.extract(i);
			//if (i == 0)
			//	System.out.printf("PneumaticExtractorPart.extractFromInventory: slot %s contains %s\n",
			//		i, stack);
			if (stack != null && stack.stackSize >= minStack) {
				int n = min(stack.stackSize, maxStack);
				//System.out.printf("PneumaticExtractorPart.extractFromInventory: splitting %s from %s\n",
				//	n, stack);
				ItemStack extracted = stack.splitStack(n);
				//System.out.printf("PneumaticExtractorPart.extractFromInventory: results are %s and %s\n",
				//	extracted, stack);
				if (stack.stackSize == 0)
					stack = null;
				inv.set(i, stack);
				return extracted;
			}
		}
		return null;
	}

	@Override
	public boolean activate(EntityPlayer player, MovingObjectPosition hit, ItemStack stack) {
		if (!world().isRemote)
			ProjectBlue.mod.openGui(player, PBGui.PneumaticExtractor, (TileEntity)tile(), side);
		return true;
	}
	
	@Override
	public void onPartChanged(TMultiPart part) {
		onNeighborChanged();
	}
	
	@Override
	public void onNeighborChanged() {
		boolean newInput = getAllRedstoneInputs() > 0;
		//System.out.printf("PneumaticExtractorPart.onNeighborChanged: newInput = %s\n", newInput);
		if (!redstoneInput && newInput)
			performExtraction();
		redstoneInput = newInput;
		markDirty();
	}
	
	// ---------- PBWiredFacePart ----------
	
//	@Override
//	public boolean canConnectTo(IConnectable part, int r) {
//		return part instanceof IRedwireEmitter;
//	}
	
	@Override
	public boolean canConnectToRedstone(int edge) {
		return true;
	}

	// ---------- IRedwireEmitter ----------
	
	public int getRedwireSignal(int side) {
		return 0;
	}
	

//	// ---------- IRestonePart ----------
//	
//	@Override
//	public int strongPowerLevel(int side) {
//		return 0;
//	}
//	
//	public int weakPowerLevel(int side) {
//		return 0;
//	}
//	
//	@Override
//	public boolean canConnectRedstone(int side) {
//		return true;
//	}
//
//	// ---------- IFaceRestonePart ----------
//	
//	@Override
//	public int getFace() {
//		return side;
//	}

	// ---------- Rendering ----------

	@Override
	public void registerIcons(IIconRegister reg) {
		textures = ProjectBlue.mod.getTextures(reg,
			"pneumatic_extractor", "pneumatic_extractor_back");
		model = ProjectBlue.mod.getModel("pneumatic_extractor");
	}

	@Override
	public boolean renderStatic(Trans3 t, int pass, IPBRenderer r) {
		model.render(t, r, textures);
		return false;
	}

}
