//------------------------------------------------------------------------------------------------
//
//   Project Blue - Pneumatic Tube Renderer
//
//------------------------------------------------------------------------------------------------

package gcewing.projectblue;

import static java.lang.Math.*;

import net.minecraft.client.*;
import net.minecraft.client.renderer.*;
import net.minecraft.client.renderer.entity.*;
import net.minecraft.client.renderer.texture.*;
import net.minecraft.entity.item.*;
import net.minecraft.item.*;
import net.minecraftforge.common.util.*;
import codechicken.multipart.*;

import gcewing.projectblue.PneumaticTubePart.Payload;

public class PneumaticTubeRenderer {

	static PBTexture pipeTexture, pipeOneWayTexture, pipePaintTexture, payloadTexture;
	static PBModel sideModel, endModel, payloadModel;
	static PayloadEntityItem payloadEntityItem = new PayloadEntityItem();
	static PayloadRenderItem renderItem = new PayloadRenderItem();
	
	static void init() {
		sideModel = ProjectBlue.getModel("pneumatic_tube_side");
		endModel = ProjectBlue.getModel("pneumatic_tube_end");
		payloadModel = ProjectBlue.getModel("pneumatic_payload");
	}
	
	static void registerIcons(IIconRegister reg) {
		pipeTexture = getTexture(reg, "pneumatic_tube");
		pipeOneWayTexture = getTexture(reg, "pneumatic_oneway");
		pipePaintTexture = getTexture(reg, "pneumatic_paint");
		payloadTexture = getTexture(reg, "pneumatic_payload");
	}
	
	static PBTexture getTexture(IIconRegister reg, String name) {
		return new PBTexture(ProjectBlue.mod.getIcon(reg, name));
	}
	
	public static void renderStaticInWorld(PneumaticTubePart part, double x, double y, double z) {
		renderTube(part, x, y, z, new PBStaticRenderer(part));
	}
	
	public static void renderDynamicInWorld(PneumaticTubePart part, double x, double y, double z, double frame) {
		renderPayloads(part, x, y, z, frame, new PBDynamicRenderer());
	}
	
	public static void renderItem(PneumaticTubePart part) {
		renderTube(part, 0, 0, 0, new PBDynamicRenderer());
	}
	
	static void renderTube(PneumaticTubePart part, double x, double y, double z, IPBRenderer pbr) {
		PBModel model;
		PBTexture tex;
		Trans3 t = new Trans3(x + 0.5, y + 0.5, z + 0.5);
		for (int i = 0; i < 6; i++) {
			Trans3 ts = t.side(i);
			if (part.isConnectedOnSide(i))
				model = sideModel;
			else
				model = endModel;
			if (part.outputSide == i)
				tex = pipeOneWayTexture;
			else
				tex = pipeTexture;
			model.render(ts, pbr, tex);
			if (part.outputSide == -1) {
				byte tag = part.tags[i];
				if (tag >= 0) {
					int c = BaseColorUtils.colors[tag];
					model.render(ts, pbr, pipePaintTexture.tinted(c));
				}
			}
		}
	}
	
	static void renderPayloads(PneumaticTubePart part, double x, double y, double z, double frame, IPBRenderer pbr) {
		RenderManager.instance.renderEngine = Minecraft.getMinecraft().getTextureManager();
		for (int i = 0; i < 6; i++) {
			Payload payload = part.payloads[i];
			if (payload != null) {
				ForgeDirection d = ForgeDirection.getOrientation(i);
				payloadEntityItem.stack = payload.stack;
				double p = 1 - payload.position;
				//p =- frame * payload.velocity;
				double xp = x + 0.5 * (1 + d.offsetX * p);
				double yp = y + 0.5 * (1 + d.offsetY * p);
				double zp = z + 0.5 * (1 + d.offsetZ * p);
				System.out.printf("PneumaticTubeRenderer.renderPayloads: side %s pos %.3f p = %.3f coords = (%.2f,%.2f,%.2f)\n",
					i, payload.position, p, xp, yp, zp);
				renderItem.doRender(payloadEntityItem, xp, yp, zp, 0, 0);
				if (payload.tag >= 0) {
					//System.out.printf("PneumaticTubeRenderer.renderPayloads: tag %s at (%.2f, %.2f, %.2f)\n",
					//	payload.tag, xp, yp, zp);
					PBTexture tex = payloadTexture.tinted(BaseColorUtils.colors[payload.tag]);
					payloadModel.render(new Trans3(xp, yp, zp), pbr, tex);
				}
			}
		}
	}

}

//------------------------------------------------------------------------------------------------

class PayloadEntityItem extends EntityItem {

	public ItemStack stack;

	public PayloadEntityItem() {
		super(null);
		hoverStart = 0; //(float)PI/4;
	}
	
	@Override
	public ItemStack getEntityItem() {
		return stack;
	}

}

//------------------------------------------------------------------------------------------------

class PayloadRenderItem extends RenderItem {

	public PayloadRenderItem() {
		setRenderManager(RenderManager.instance);
	}
	
	public RenderManager getRenderManager() {
		return renderManager;
	}
	
	@Override
	public boolean shouldSpreadItems() {
		return false;
	}
	
	@Override
	public boolean shouldBob() {
		return false;
	}
	
	@Override
	public byte getMiniBlockCount(ItemStack stack, byte original) {
		return 1;
	}
	
	@Override
	public byte getMiniItemCount(ItemStack stack, byte original) {
		return 1;
	}

}
